import { Request, Response } from "express";
import prisma, { Prisma } from "@server/prisma";
import { datatableRequest } from "@server/controllers/guild/api/dt.module";
import { logger } from "@server/../log";

export const get = async (request: Request, response: Response) => {
    if (!request.query.id) {
        response.status(400).json({ error: "missing 'id' query" });
        return;
    }

    const style = await prisma.messageStyle.findUnique({
        where: { id: Number(request.query.id) },
    });

    if (!style) {
        response.status(404).json({ message: "no result found" });
        return;
    }

    response.json(style);
};

export const post = async (request: Request, response: Response) => {
    const guildId = request.params.guildId;
    const {
        name,
        show_author,
        show_image,
        show_thumbnail,
        show_footer,
        show_timestamp,
        colour,
        title_mutator,
        description_mutator
    } = request.body;

    logger.debug("Style Post", request.body);

    let style;

    try {
        style = await prisma.messageStyle.create({
            data: {
                name: name,
                guild_id: guildId,
                show_author: show_author === "on",
                show_image: show_image === "on",
                show_thumbnail: show_thumbnail === "on",
                show_footer: show_footer === "on",
                show_timestamp: show_timestamp === "on",
                colour: colour,
                title_mutator: title_mutator || null,
                description_mutator: description_mutator || null
             }
        });
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(201).json(style);
};

export const patch = async (request: Request, response: Response) => {
    const guildId = request.params.guildId;
    const {
        id,
        name,
        show_author,
        show_image,
        show_thumbnail,
        show_footer,
        show_timestamp,
        colour,
        title_mutator,
        description_mutator
    } = request.body;

    let style;

    try {
        style = await prisma.messageStyle.update({
            where: { id: Number(id) },
            data: {
                name: name,
                guild_id: guildId,
                show_author: show_author === "on",
                show_image: show_image === "on",
                show_thumbnail: show_thumbnail === "on",
                show_footer: show_footer === "on",
                show_timestamp: show_timestamp === "on",
                colour: colour,
                title_mutator: title_mutator || null,
                description_mutator: description_mutator || null
             }
        });
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(201).json(style);
};

export const del = async (request: Request, response: Response) => {
    let { ids } = request.body;
    const guildId = request.params.guildId;

    if (!ids || !Array.isArray(ids)) {
        response.status(400).json({ error: "invalid request body" });
        return;
    }

    ids = ids.map(id => Number(id));

    try {
        await prisma.messageStyle.deleteMany({ where: {
            id: { in: ids },
            guild_id: guildId
        }});
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(204).json(null);
};

export const datatable = async (request: Request, response: Response) => {
    return await datatableRequest(
        request,
        response,
        prisma.messageStyle,
        [{ updated_at: "desc" }, { id: "asc" }],
        { },
        { guild_id: request.params.guildId }  // TODO: verify authenticated user can access this guild
    );
};

export const select = async (request: Request, response: Response) => {
    const guildId = request.params.guildId;
    const { search } = request.query;

    const data = await prisma.messageStyle.findMany({
        where: {
            guild_id: guildId,
            name: { contains: `${search}` }
        }
    });

    // Preline Bug: https://github.com/htmlstreamofficial/preline/issues/567
    // The returned data must have a "title" key, otherwise the advanced
    // select component with 'tags' mode will have no title, regardless of
    // mapping.
    const modifiedResults = data.map(filter => ({
        ...filter,
        title: filter.name
    }));

    response.json(modifiedResults);
};

export default { get, post, patch, del, datatable, select };